@@ -5,16 +5,14 @@
 #ifdef _OPENMP
 #include <omp.h>
 #endif
-#include <pythonic/include/types/int.hpp>
 #include <pythonic/include/types/numpy_texpr.hpp>
-#include <pythonic/include/types/int32.hpp>
 #include <pythonic/include/types/ndarray.hpp>
+#include <pythonic/include/types/int.hpp>
+#include <pythonic/include/types/int32.hpp>
 #include <pythonic/types/int32.hpp>
 #include <pythonic/types/ndarray.hpp>
-#include <pythonic/types/numpy_texpr.hpp>
 #include <pythonic/types/int.hpp>
-#include <pythonic/include/builtins/False.hpp>
-#include <pythonic/include/builtins/True.hpp>
+#include <pythonic/types/numpy_texpr.hpp>
 #include <pythonic/include/builtins/getattr.hpp>
 #include <pythonic/include/builtins/pythran/and_.hpp>
 #include <pythonic/include/builtins/range.hpp>
@@ -33,8 +31,6 @@
 #include <pythonic/include/operator_/not_.hpp>
 #include <pythonic/include/types/slice.hpp>
 #include <pythonic/include/types/str.hpp>
-#include <pythonic/builtins/False.hpp>
-#include <pythonic/builtins/True.hpp>
 #include <pythonic/builtins/getattr.hpp>
 #include <pythonic/builtins/pythran/and_.hpp>
 #include <pythonic/builtins/range.hpp>
@@ -85,6 +81,7 @@ namespace __pythran__group_columns
     }  
     ;
     template <typename argument_type0 , typename argument_type1 , typename argument_type2 , typename argument_type3 >
+    inline
     typename type<argument_type0, argument_type1, argument_type2, argument_type3>::result_type operator()(argument_type0&& m, argument_type1&& n, argument_type2&& indices, argument_type3&& indptr) const
     ;
   }  ;
@@ -118,10 +115,12 @@ namespace __pythran__group_columns
     }  
     ;
     template <typename argument_type0 , typename argument_type1 , typename argument_type2 >
+    inline
     typename type<argument_type0, argument_type1, argument_type2>::result_type operator()(argument_type0&& m, argument_type1&& n, argument_type2&& A) const
     ;
   }  ;
   template <typename argument_type0 , typename argument_type1 , typename argument_type2 , typename argument_type3 >
+  inline
   typename group_sparse::type<argument_type0, argument_type1, argument_type2, argument_type3>::result_type group_sparse::operator()(argument_type0&& m, argument_type1&& n, argument_type2&& indices, argument_type3&& indptr) const
   {
     typedef typename std::remove_cv<typename std::remove_reference<decltype(pythonic::numpy::functor::ones{})>::type>::type __type0;
@@ -169,52 +168,47 @@ namespace __pythran__group_columns
     typedef indexable<__type52> __type53;
     typedef typename __combined<__type41,__type53>::type __type54;
     typedef container<typename std::remove_reference<__type12>::type> __type55;
-    typename pythonic::assignable<typename std::remove_cv<typename std::iterator_traits<typename std::remove_reference<__type20>::type::iterator>::value_type>::type>::type j;
-    typename pythonic::assignable<typename std::remove_cv<typename std::iterator_traits<typename std::remove_reference<__type7>::type::iterator>::value_type>::type>::type i;
-    typename pythonic::assignable<typename std::remove_cv<typename std::iterator_traits<typename std::remove_reference<__type50>::type::iterator>::value_type>::type>::type k__;
-    typename pythonic::assignable<typename std::remove_cv<typename std::iterator_traits<typename std::remove_reference<__type37>::type::iterator>::value_type>::type>::type k;
-    typename pythonic::assignable<typename std::remove_cv<typename std::iterator_traits<typename std::remove_reference<__type50>::type::iterator>::value_type>::type>::type k_;
     typename pythonic::assignable<typename __combined<__type23,__type16,__type9,__type16,__type22>::type>::type groups = pythonic::operator_::neg(pythonic::numpy::functor::ones{}(n, pythonic::numpy::functor::intp{}));
     typename pythonic::assignable<typename __combined<__type14,__type12>::type>::type current_group = 0L;
     typename pythonic::assignable<typename __combined<__type54,__type55,__type40,__type53>::type>::type union_ = pythonic::numpy::functor::empty{}(m, pythonic::numpy::functor::intp{});
     {
-      long  __target139877265928976 = n;
-      for (long  i=0L; i < __target139877265928976; i += 1L)
+      long  __target139767940641696 = n;
+      for (long  i=0L; i < __target139767940641696; i += 1L)
       {
-        if (pythonic::operator_::ge(groups.fast(i), 0L))
+        if (pythonic::operator_::ge(groups[i], 0L))
         {
           continue;
         }
-        groups.fast(i) = current_group;
-        typename pythonic::lazy<decltype(pythonic::builtins::True)>::type all_grouped = pythonic::builtins::True;
+        groups[i] = current_group;
+        typename pythonic::lazy<decltype(true)>::type all_grouped = true;
         pythonic::numpy::ndarray::functor::fill{}(union_, 0L);
         {
-          long  __target139877265937120 = indptr.fast(pythonic::operator_::add(i, 1L));
-          for (long  k=indptr.fast(i); k < __target139877265937120; k += 1L)
+          long  __target139767940644672 = indptr[pythonic::operator_::add(i, 1L)];
+          for (long  k=indptr[i]; k < __target139767940644672; k += 1L)
           {
             union_[indices[k]] = 1L;
           }
         }
         {
-          long  __target139877265937600 = std::get<0>(pythonic::builtins::getattr(pythonic::types::attr::SHAPE{}, groups));
-          for (long  j=0L; j < __target139877265937600; j += 1L)
+          long  __target139767940747664 = std::get<0>(pythonic::builtins::getattr(pythonic::types::attr::SHAPE{}, groups));
+          for (long  j=0L; j < __target139767940747664; j += 1L)
           {
-            if (pythonic::operator_::lt(groups.fast(j), 0L))
+            if (pythonic::operator_::lt(groups[j], 0L))
             {
-              all_grouped = pythonic::builtins::False;
+              all_grouped = false;
             }
             else
             {
               continue;
             }
-            typename pythonic::lazy<decltype(pythonic::builtins::False)>::type intersect = pythonic::builtins::False;
+            typename pythonic::lazy<decltype(false)>::type intersect = false;
             {
-              long  __target139877265789568 = indptr.fast(pythonic::operator_::add(j, 1L));
-              for (long  k_=indptr.fast(j); k_ < __target139877265789568; k_ += 1L)
+              long  __target139767940750064 = indptr[pythonic::operator_::add(j, 1L)];
+              for (long  k_=indptr[j]; k_ < __target139767940750064; k_ += 1L)
               {
                 if (pythonic::operator_::eq(union_[indices[k_]], 1L))
                 {
-                  intersect = pythonic::builtins::True;
+                  intersect = true;
                   break;
                 }
               }
@@ -222,13 +216,13 @@ namespace __pythran__group_columns
             if (pythonic::operator_::not_(intersect))
             {
               {
-                long  __target139877265802576 = indptr.fast(pythonic::operator_::add(j, 1L));
-                for (long  k__=indptr.fast(j); k__ < __target139877265802576; k__ += 1L)
+                long  __target139767940745488 = indptr[pythonic::operator_::add(j, 1L)];
+                for (long  k__=indptr[j]; k__ < __target139767940745488; k__ += 1L)
                 {
                   union_[indices[k__]] = 1L;
                 }
               }
-              groups.fast(j) = current_group;
+              groups[j] = current_group;
             }
           }
         }
@@ -245,6 +239,7 @@ namespace __pythran__group_columns
     return groups;
   }
   template <typename argument_type0 , typename argument_type1 , typename argument_type2 >
+  inline
   typename group_dense::type<argument_type0, argument_type1, argument_type2>::result_type group_dense::operator()(argument_type0&& m, argument_type1&& n, argument_type2&& A) const
   {
     typedef typename std::remove_cv<typename std::remove_reference<decltype(pythonic::numpy::functor::ones{})>::type>::type __type0;
@@ -279,53 +274,49 @@ namespace __pythran__group_columns
     typedef decltype(pythonic::operator_::add(std::declval<__type28>(), std::declval<__type32>())) __type33;
     typedef typename __combined<__type28,__type33>::type __type34;
     typedef decltype(std::declval<__type30>()[std::declval<__type8>()]) __type37;
-    typedef decltype(std::declval<__type5>()(std::declval<__type27>())) __type39;
-    typename pythonic::assignable<typename std::remove_cv<typename std::iterator_traits<typename std::remove_reference<__type20>::type::iterator>::value_type>::type>::type j;
-    typename pythonic::assignable<typename std::remove_cv<typename std::iterator_traits<typename std::remove_reference<__type7>::type::iterator>::value_type>::type>::type i;
-    typename pythonic::assignable<typename std::remove_cv<typename std::iterator_traits<typename std::remove_reference<__type39>::type::iterator>::value_type>::type>::type k;
     typename pythonic::assignable_noescape<decltype(pythonic::builtins::getattr(pythonic::types::attr::T{}, A))>::type B = pythonic::builtins::getattr(pythonic::types::attr::T{}, A);
     typename pythonic::assignable<typename __combined<__type23,__type16,__type9,__type16,__type22>::type>::type groups = pythonic::operator_::neg(pythonic::numpy::functor::ones{}(n, pythonic::numpy::functor::intp{}));
     typename pythonic::assignable<typename __combined<__type14,__type12>::type>::type current_group = 0L;
     typename pythonic::assignable<typename __combined<__type34,__type37,__type32>::type>::type union_ = pythonic::numpy::functor::empty{}(m, pythonic::numpy::functor::intp{});
     {
-      long  __target139877268977600 = n;
-      for (long  i=0L; i < __target139877268977600; i += 1L)
+      long  __target139767936400784 = n;
+      for (long  i=0L; i < __target139767936400784; i += 1L)
       {
-        if (pythonic::operator_::ge(groups.fast(i), 0L))
+        if (pythonic::operator_::ge(groups[i], 0L))
         {
           continue;
         }
-        groups.fast(i) = current_group;
-        typename pythonic::lazy<decltype(pythonic::builtins::True)>::type all_grouped = pythonic::builtins::True;
-        union_[pythonic::types::contiguous_slice(pythonic::builtins::None,pythonic::builtins::None)] = B.fast(i);
+        groups[i] = current_group;
+        typename pythonic::lazy<decltype(true)>::type all_grouped = true;
+        union_[pythonic::types::contiguous_slice(pythonic::builtins::None,pythonic::builtins::None)] = B[i];
         {
-          long  __target139877268895824 = std::get<0>(pythonic::builtins::getattr(pythonic::types::attr::SHAPE{}, groups));
-          for (long  j=0L; j < __target139877268895824; j += 1L)
+          long  __target139767937798832 = std::get<0>(pythonic::builtins::getattr(pythonic::types::attr::SHAPE{}, groups));
+          for (long  j=0L; j < __target139767937798832; j += 1L)
           {
-            if (pythonic::operator_::lt(groups.fast(j), 0L))
+            if (pythonic::operator_::lt(groups[j], 0L))
             {
-              all_grouped = pythonic::builtins::False;
+              all_grouped = false;
             }
             else
             {
               continue;
             }
-            typename pythonic::lazy<decltype(pythonic::builtins::False)>::type intersect = pythonic::builtins::False;
+            typename pythonic::lazy<decltype(false)>::type intersect = false;
             {
-              long  __target139877269002032 = m;
-              for (long  k=0L; k < __target139877269002032; k += 1L)
+              long  __target139767937800368 = m;
+              for (long  k=0L; k < __target139767937800368; k += 1L)
               {
-                if (pythonic::builtins::pythran::and_([&] () { return pythonic::operator_::gt(union_.fast(k), 0L); }, [&] () { return pythonic::operator_::gt(B.fast(pythonic::types::make_tuple(j, k)), 0L); }))
+                if (pythonic::builtins::pythran::and_([&] () { return pythonic::operator_::gt(union_[k], 0L); }, [&] () { return pythonic::operator_::gt(B[pythonic::types::make_tuple(j, k)], 0L); }))
                 {
-                  intersect = pythonic::builtins::True;
+                  intersect = true;
                   break;
                 }
               }
             }
             if (pythonic::operator_::not_(intersect))
             {
-              union_ += B.fast(j);
-              groups.fast(j) = current_group;
+              union_ += B[j];
+              groups[j] = current_group;
             }
           }
         }
@@ -344,7 +335,8 @@ namespace __pythran__group_columns
 }
 #include <pythonic/python/exception_handler.hpp>
 #ifdef ENABLE_PYTHON_MODULE
-typename __pythran__group_columns::group_sparse::type<long, long, pythonic::types::ndarray<long,pythonic::types::pshape<long>>, pythonic::types::ndarray<long,pythonic::types::pshape<long>>>::result_type group_sparse0(long&& m, long&& n, pythonic::types::ndarray<long,pythonic::types::pshape<long>>&& indices, pythonic::types::ndarray<long,pythonic::types::pshape<long>>&& indptr) 
+inline
+typename __pythran__group_columns::group_sparse::type<long, long, pythonic::types::numpy_gexpr<pythonic::types::ndarray<long,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>, pythonic::types::numpy_gexpr<pythonic::types::ndarray<long,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>>::result_type group_sparse0(long&& m, long&& n, pythonic::types::numpy_gexpr<pythonic::types::ndarray<long,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>&& indices, pythonic::types::numpy_gexpr<pythonic::types::ndarray<long,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>&& indptr) 
 {
   
                             PyThreadState *_save = PyEval_SaveThread();
@@ -359,7 +351,8 @@ typename __pythran__group_columns::group_sparse::type<long, long, pythonic::type
                             }
                             ;
 }
-typename __pythran__group_columns::group_sparse::type<long, long, pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>, pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>>::result_type group_sparse1(long&& m, long&& n, pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>&& indices, pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>&& indptr) 
+inline
+typename __pythran__group_columns::group_sparse::type<long, long, pythonic::types::numpy_gexpr<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>, pythonic::types::numpy_gexpr<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>>::result_type group_sparse1(long&& m, long&& n, pythonic::types::numpy_gexpr<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>&& indices, pythonic::types::numpy_gexpr<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>&& indptr) 
 {
   
                             PyThreadState *_save = PyEval_SaveThread();
@@ -374,6 +367,39 @@ typename __pythran__group_columns::group_sparse::type<long, long, pythonic::type
                             }
                             ;
 }
+inline
+typename __pythran__group_columns::group_sparse::type<long, long, pythonic::types::ndarray<long,pythonic::types::pshape<long>>, pythonic::types::ndarray<long,pythonic::types::pshape<long>>>::result_type group_sparse2(long&& m, long&& n, pythonic::types::ndarray<long,pythonic::types::pshape<long>>&& indices, pythonic::types::ndarray<long,pythonic::types::pshape<long>>&& indptr) 
+{
+  
+                            PyThreadState *_save = PyEval_SaveThread();
+                            try {
+                                auto res = __pythran__group_columns::group_sparse()(m, n, indices, indptr);
+                                PyEval_RestoreThread(_save);
+                                return res;
+                            }
+                            catch(...) {
+                                PyEval_RestoreThread(_save);
+                                throw;
+                            }
+                            ;
+}
+inline
+typename __pythran__group_columns::group_sparse::type<long, long, pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>, pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>>::result_type group_sparse3(long&& m, long&& n, pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>&& indices, pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>&& indptr) 
+{
+  
+                            PyThreadState *_save = PyEval_SaveThread();
+                            try {
+                                auto res = __pythran__group_columns::group_sparse()(m, n, indices, indptr);
+                                PyEval_RestoreThread(_save);
+                                return res;
+                            }
+                            catch(...) {
+                                PyEval_RestoreThread(_save);
+                                throw;
+                            }
+                            ;
+}
+inline
 typename __pythran__group_columns::group_dense::type<long, long, pythonic::types::ndarray<long,pythonic::types::pshape<long,long>>>::result_type group_dense0(long&& m, long&& n, pythonic::types::ndarray<long,pythonic::types::pshape<long,long>>&& A) 
 {
   
@@ -389,6 +415,7 @@ typename __pythran__group_columns::group_dense::type<long, long, pythonic::types
                             }
                             ;
 }
+inline
 typename __pythran__group_columns::group_dense::type<long, long, pythonic::types::numpy_texpr<pythonic::types::ndarray<long,pythonic::types::pshape<long,long>>>>::result_type group_dense1(long&& m, long&& n, pythonic::types::numpy_texpr<pythonic::types::ndarray<long,pythonic::types::pshape<long,long>>>&& A) 
 {
   
@@ -404,6 +431,7 @@ typename __pythran__group_columns::group_dense::type<long, long, pythonic::types
                             }
                             ;
 }
+inline
 typename __pythran__group_columns::group_dense::type<long, long, pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long,long>>>::result_type group_dense2(long&& m, long&& n, pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long,long>>&& A) 
 {
   
@@ -419,6 +447,7 @@ typename __pythran__group_columns::group_dense::type<long, long, pythonic::types
                             }
                             ;
 }
+inline
 typename __pythran__group_columns::group_dense::type<long, long, pythonic::types::numpy_texpr<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long,long>>>>::result_type group_dense3(long&& m, long&& n, pythonic::types::numpy_texpr<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long,long>>>&& A) 
 {
   
@@ -439,12 +468,13 @@ static PyObject *
 __pythran_wrap_group_sparse0(PyObject *self, PyObject *args, PyObject *kw)
 {
     PyObject* args_obj[4+1];
+    
     char const* keywords[] = {"m", "n", "indices", "indptr",  nullptr};
     if(! PyArg_ParseTupleAndKeywords(args, kw, "OOOO",
                                      (char**)keywords , &args_obj[0], &args_obj[1], &args_obj[2], &args_obj[3]))
         return nullptr;
-    if(is_convertible<long>(args_obj[0]) && is_convertible<long>(args_obj[1]) && is_convertible<pythonic::types::ndarray<long,pythonic::types::pshape<long>>>(args_obj[2]) && is_convertible<pythonic::types::ndarray<long,pythonic::types::pshape<long>>>(args_obj[3]))
-        return to_python(group_sparse0(from_python<long>(args_obj[0]), from_python<long>(args_obj[1]), from_python<pythonic::types::ndarray<long,pythonic::types::pshape<long>>>(args_obj[2]), from_python<pythonic::types::ndarray<long,pythonic::types::pshape<long>>>(args_obj[3])));
+    if(is_convertible<long>(args_obj[0]) && is_convertible<long>(args_obj[1]) && is_convertible<pythonic::types::numpy_gexpr<pythonic::types::ndarray<long,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>>(args_obj[2]) && is_convertible<pythonic::types::numpy_gexpr<pythonic::types::ndarray<long,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>>(args_obj[3]))
+        return to_python(group_sparse0(from_python<long>(args_obj[0]), from_python<long>(args_obj[1]), from_python<pythonic::types::numpy_gexpr<pythonic::types::ndarray<long,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>>(args_obj[2]), from_python<pythonic::types::numpy_gexpr<pythonic::types::ndarray<long,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>>(args_obj[3])));
     else {
         return nullptr;
     }
@@ -454,12 +484,45 @@ static PyObject *
 __pythran_wrap_group_sparse1(PyObject *self, PyObject *args, PyObject *kw)
 {
     PyObject* args_obj[4+1];
+    
+    char const* keywords[] = {"m", "n", "indices", "indptr",  nullptr};
+    if(! PyArg_ParseTupleAndKeywords(args, kw, "OOOO",
+                                     (char**)keywords , &args_obj[0], &args_obj[1], &args_obj[2], &args_obj[3]))
+        return nullptr;
+    if(is_convertible<long>(args_obj[0]) && is_convertible<long>(args_obj[1]) && is_convertible<pythonic::types::numpy_gexpr<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>>(args_obj[2]) && is_convertible<pythonic::types::numpy_gexpr<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>>(args_obj[3]))
+        return to_python(group_sparse1(from_python<long>(args_obj[0]), from_python<long>(args_obj[1]), from_python<pythonic::types::numpy_gexpr<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>>(args_obj[2]), from_python<pythonic::types::numpy_gexpr<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>,pythonic::types::normalized_slice>>(args_obj[3])));
+    else {
+        return nullptr;
+    }
+}
+
+static PyObject *
+__pythran_wrap_group_sparse2(PyObject *self, PyObject *args, PyObject *kw)
+{
+    PyObject* args_obj[4+1];
+    
+    char const* keywords[] = {"m", "n", "indices", "indptr",  nullptr};
+    if(! PyArg_ParseTupleAndKeywords(args, kw, "OOOO",
+                                     (char**)keywords , &args_obj[0], &args_obj[1], &args_obj[2], &args_obj[3]))
+        return nullptr;
+    if(is_convertible<long>(args_obj[0]) && is_convertible<long>(args_obj[1]) && is_convertible<pythonic::types::ndarray<long,pythonic::types::pshape<long>>>(args_obj[2]) && is_convertible<pythonic::types::ndarray<long,pythonic::types::pshape<long>>>(args_obj[3]))
+        return to_python(group_sparse2(from_python<long>(args_obj[0]), from_python<long>(args_obj[1]), from_python<pythonic::types::ndarray<long,pythonic::types::pshape<long>>>(args_obj[2]), from_python<pythonic::types::ndarray<long,pythonic::types::pshape<long>>>(args_obj[3])));
+    else {
+        return nullptr;
+    }
+}
+
+static PyObject *
+__pythran_wrap_group_sparse3(PyObject *self, PyObject *args, PyObject *kw)
+{
+    PyObject* args_obj[4+1];
+    
     char const* keywords[] = {"m", "n", "indices", "indptr",  nullptr};
     if(! PyArg_ParseTupleAndKeywords(args, kw, "OOOO",
                                      (char**)keywords , &args_obj[0], &args_obj[1], &args_obj[2], &args_obj[3]))
         return nullptr;
     if(is_convertible<long>(args_obj[0]) && is_convertible<long>(args_obj[1]) && is_convertible<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>>(args_obj[2]) && is_convertible<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>>(args_obj[3]))
-        return to_python(group_sparse1(from_python<long>(args_obj[0]), from_python<long>(args_obj[1]), from_python<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>>(args_obj[2]), from_python<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>>(args_obj[3])));
+        return to_python(group_sparse3(from_python<long>(args_obj[0]), from_python<long>(args_obj[1]), from_python<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>>(args_obj[2]), from_python<pythonic::types::ndarray<npy_int32,pythonic::types::pshape<long>>>(args_obj[3])));
     else {
         return nullptr;
     }
@@ -469,6 +532,7 @@ static PyObject *
 __pythran_wrap_group_dense0(PyObject *self, PyObject *args, PyObject *kw)
 {
     PyObject* args_obj[3+1];
+    
     char const* keywords[] = {"m", "n", "A",  nullptr};
     if(! PyArg_ParseTupleAndKeywords(args, kw, "OOO",
                                      (char**)keywords , &args_obj[0], &args_obj[1], &args_obj[2]))
@@ -484,6 +548,7 @@ static PyObject *
 __pythran_wrap_group_dense1(PyObject *self, PyObject *args, PyObject *kw)
 {
     PyObject* args_obj[3+1];
+    
     char const* keywords[] = {"m", "n", "A",  nullptr};
     if(! PyArg_ParseTupleAndKeywords(args, kw, "OOO",
                                      (char**)keywords , &args_obj[0], &args_obj[1], &args_obj[2]))
@@ -499,6 +564,7 @@ static PyObject *
 __pythran_wrap_group_dense2(PyObject *self, PyObject *args, PyObject *kw)
 {
     PyObject* args_obj[3+1];
+    
     char const* keywords[] = {"m", "n", "A",  nullptr};
     if(! PyArg_ParseTupleAndKeywords(args, kw, "OOO",
                                      (char**)keywords , &args_obj[0], &args_obj[1], &args_obj[2]))
@@ -514,6 +580,7 @@ static PyObject *
 __pythran_wrap_group_dense3(PyObject *self, PyObject *args, PyObject *kw)
 {
     PyObject* args_obj[3+1];
+    
     char const* keywords[] = {"m", "n", "A",  nullptr};
     if(! PyArg_ParseTupleAndKeywords(args, kw, "OOO",
                                      (char**)keywords , &args_obj[0], &args_obj[1], &args_obj[2]))
@@ -540,8 +607,18 @@ if(PyObject* obj = __pythran_wrap_group_sparse1(self, args, kw))
     return obj;
 PyErr_Clear();
 
+
+if(PyObject* obj = __pythran_wrap_group_sparse2(self, args, kw))
+    return obj;
+PyErr_Clear();
+
+
+if(PyObject* obj = __pythran_wrap_group_sparse3(self, args, kw))
+    return obj;
+PyErr_Clear();
+
                 return pythonic::python::raise_invalid_argument(
-                               "group_sparse", "\n""    - group_sparse(int, int, int[:], int[:])\n""    - group_sparse(int, int, int32[:], int32[:])", args, kw);
+                               "group_sparse", "\n""    - group_sparse(int, int, int[::], int[::])\n""    - group_sparse(int, int, int32[::], int32[::])\n""    - group_sparse(int, int, int[:], int[:])\n""    - group_sparse(int, int, int32[:], int32[:])", args, kw);
                 });
             }
 
@@ -582,7 +659,7 @@ static PyMethodDef Methods[] = {
     "group_sparse",
     (PyCFunction)__pythran_wrapall_group_sparse,
     METH_VARARGS | METH_KEYWORDS,
-    "Supported prototypes:\n""\n""    - group_sparse(int, int, int[:], int[:])\n""    - group_sparse(int, int, int32[:], int32[:])"},{
+    "Supported prototypes:\n""\n""    - group_sparse(int, int, int[::], int[::])\n""    - group_sparse(int, int, int32[::], int32[::])\n""    - group_sparse(int, int, int[:], int[:])\n""    - group_sparse(int, int, int32[:], int32[:])"},{
     "group_dense",
     (PyCFunction)__pythran_wrapall_group_dense,
     METH_VARARGS | METH_KEYWORDS,
@@ -613,8 +690,10 @@ PyMODINIT_FUNC
 PYTHRAN_MODULE_INIT(_group_columns)(void)
 #ifndef _WIN32
 __attribute__ ((visibility("default")))
+#if defined(GNUC) && !defined(__clang__)
 __attribute__ ((externally_visible))
 #endif
+#endif
 ;
 PyMODINIT_FUNC
 PYTHRAN_MODULE_INIT(_group_columns)(void) {
@@ -630,9 +709,9 @@ PYTHRAN_MODULE_INIT(_group_columns)(void) {
     if(! theModule)
         PYTHRAN_RETURN;
     PyObject * theDoc = Py_BuildValue("(sss)",
-                                      "0.9.11",
-                                      "2021-11-23 21:33:21.538180",
-                                      "ac77c0cfb3b65ea485951fd3a308c3ac40dd2c1905835b2982a66e23bfff0ebc");
+                                      "0.10.0",
+                                      "2022-02-04 07:45:35.848004",
+                                      "bbd9c32ae0cc40f86da54c6bc74a72b8cc51ed42efd9d0a4f45d3648fc5482c3");
     if(! theDoc)
         PYTHRAN_RETURN;
     PyModule_AddObject(theModule,
